using System;
using System.Collections.Generic;

namespace StudioX.Application.Services.Dto
{
    /// <summary>
    ///     Implements <see cref="IListResult{T}" />.
    /// </summary>
    /// <typeparam name="T">Type of the items in the <see cref="Items" /> list</typeparam>
    [Serializable]
    public class ListResultDto<T> : IListResult<T>
    {
        /// <summary>
        ///     List of items.
        /// </summary>
        public IReadOnlyList<T> Items
        {
            get => items ?? (items = new List<T>());
            set => items = value;
        }

        private IReadOnlyList<T> items;

        /// <summary>
        ///     Creates a new <see cref="ListResultDto{T}" /> object.
        /// </summary>
        public ListResultDto()
        {
        }

        /// <summary>
        ///     Creates a new <see cref="ListResultDto{T}" /> object.
        /// </summary>
        /// <param name="items">List of items</param>
        public ListResultDto(IReadOnlyList<T> items)
        {
            Items = items;
        }
    }
}