﻿using System;
using System.Runtime.Serialization;
using StudioX.Logging;

namespace StudioX.Authorization
{
    /// <summary>
    ///     This exception is thrown on an unauthorized request.
    /// </summary>
    [Serializable]
    public class StudioXAuthorizationException : StudioXException, IHasLogSeverity
    {
        /// <summary>
        ///     Severity of the exception.
        ///     Default: Warn.
        /// </summary>
        public LogSeverity Severity { get; set; }

        /// <summary>
        ///     Creates a new <see cref="StudioXAuthorizationException" /> object.
        /// </summary>
        public StudioXAuthorizationException()
        {
            Severity = LogSeverity.Warn;
        }

#if NET46
        /// <summary>
        ///     Creates a new <see cref="StudioXAuthorizationException" /> object.
        /// </summary>
        public StudioXAuthorizationException(SerializationInfo serializationInfo, StreamingContext context)
            : base(serializationInfo, context)
        {
        }
#endif

        /// <summary>
        ///     Creates a new <see cref="StudioXAuthorizationException" /> object.
        /// </summary>
        /// <param name="message">Exception message</param>
        public StudioXAuthorizationException(string message)
            : base(message)
        {
            Severity = LogSeverity.Warn;
        }

        /// <summary>
        ///     Creates a new <see cref="StudioXAuthorizationException" /> object.
        /// </summary>
        /// <param name="message">Exception message</param>
        /// <param name="innerException">Inner exception</param>
        public StudioXAuthorizationException(string message, Exception innerException)
            : base(message, innerException)
        {
            Severity = LogSeverity.Warn;
        }
    }
}